package com.sun.media.sound;

import java.util.ServiceLoader;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Properties;
import javax.sound.sampled.AudioPermission;

final class JSSecurityManager
{
    private JSSecurityManager() {
    }
    
    private static boolean hasSecurityManager() {
        return System.getSecurityManager() != null;
    }
    
    static void checkRecordPermission() throws SecurityException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new AudioPermission("record"));
        }
    }
    
    static void loadProperties(final Properties properties, final String s) {
        if (hasSecurityManager()) {
            try {
                AccessController.doPrivileged(new PrivilegedAction<Void>() {
                    @Override
                    public Void run() {
                        JSSecurityManager.access$000(properties, s);
                        return null;
                    }
                });
            }
            catch (Exception ex) {
                loadPropertiesImpl(properties, s);
            }
        }
        else {
            loadPropertiesImpl(properties, s);
        }
    }
    
    private static void loadPropertiesImpl(Properties properties, String child) {
        String property = System.getProperty("java.home");
        try {
            if (property == null) {
                throw new Error("Can't find java.home ??");
            }
            FileInputStream in = new FileInputStream(new File(new File(property, "lib"), child).getCanonicalPath());
            BufferedInputStream inStream = new BufferedInputStream(in);
            try {
                properties.load(inStream);
            }
            finally {
                if (in != null) {
                    in.close();
                }
            }
        }
        catch (Throwable t) {}
    }
    
    static Thread createThread(Runnable task, String name, boolean daemon, int priority, boolean b) {
        Thread thread = new Thread(task);
        if (name != null) {
            thread.setName(name);
        }
        thread.setDaemon(daemon);
        if (priority >= 0) {
            thread.setPriority(priority);
        }
        if (b) {
            thread.start();
            return thread;
        }
        return thread;
    }
    
    static synchronized <T> List<T> getProviders(final Class<T> clazz) {
        ArrayList list = new ArrayList(7);
        final Iterator<Object> iterator = (Iterator)AccessController.doPrivileged(new PrivilegedAction<Iterator<T>>() {
            @Override
            public Iterator<T> run() {
                return ServiceLoader.load(clazz).iterator();
            }
        });
        while ((boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                return Boolean.valueOf(iterator.hasNext());
            }
        })) {
            try {
                Object next = iterator.next();
                if (clazz.isInstance(next)) {
                    list.add(0, next);
                }
                else {
                    continue;
                }
            }
            catch (Throwable t) {}
        }
        return list;
    }
    
    static void access$000(Properties properties, String s) {
        loadPropertiesImpl(properties, s);
    }
}
